/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * DHCP library
 */
#ifndef _HS_DHCP_IF_H
#define _HS_DHCP_IF_H

// HS DHCP return codes
typedef enum
{
	HS_DHCP_RC_OK,							// success
	HS_DHCP_RC_BAD_PARAM,					// bad parameters
	HS_DHCP_RC_NOT_INIT,					// not initialized
	HS_DHCP_RC_ALR_INIT,					// already initialized
} hs_dhcp_rc_t;

// event callback
typedef void hs_dhcp_event_fn_t(int ev, long arg1, long arg2);

typedef void hs_dhcp_timer_cb_t(void *arg);
typedef long hs_dhcp_timer_start_t(unsigned long timeout_ms, void *arg, hs_dhcp_timer_cb_t *cb);
typedef void hs_dhcp_timer_stop_t(long timer_han);
typedef unsigned long hs_dhcp_get_ticks_fn_t(void);
typedef void hs_dhcp_log_fn_t(char *str);

typedef enum
{
	HS_DHCP_EV_CFG_UPDATE,					// Network configurtation update after renew
	HS_DHCP_EV_TIMEOUT,						// Timed out
	HS_DHCP_EV_ERROR,						// DHCP server rejected request, returned to initial state
} hs_dhcp_usr_event_t;

// update event data
typedef struct
{
	unsigned long			local_ip;		// local IP address
	unsigned long			dns_ip;			// dns server ip address
	unsigned long			router_ip;		// gateway ip address
	unsigned long			subnet_mask;	// subnext mask
	unsigned long			lease_time;		// lease time in seconds
	unsigned long			renew_time;		// renew time in seconds
	unsigned long			rebind_time;	// rebind time in seconds
} hs_dhcp_update_t;

// DHCP client configuration structure
typedef struct
{
	unsigned char 			MacAddr[6];		// own mac address
	hs_dhcp_event_fn_t 		*event_fn;		// event callback
	hs_dhcp_timer_start_t	*timer_start;	// timer start callback
	hs_dhcp_timer_stop_t	*timer_stop;	// timer stop callback
	hs_dhcp_get_ticks_fn_t	*get_rand32;	// callback to get 32 bit random number
	hs_dhcp_log_fn_t		*log_fn;		// event log function
} hsdhcp_init_t;

/*
 * Initialize DHCP client
 */
extern int HsDhcpInit(hsdhcp_init_t *init);

/*
 * De-Initialize DHCP client
 */
extern void HsDhcpCleanUp(void);

/*
 * Initiate getting network configuration
 */
extern int HsDhcpRenew(void);

/*
 * Release network configuration
 */
extern int HsDhcpRelease(void);

#endif // _HS_DHCP_IF_H

