/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * DNS library
 */
#ifndef _HS_DNS_IF_H
#define _HS_DNS_IF_H

typedef enum
{
	HS_DNS_RC_FAIL,
	HS_DNS_RC_OK,
	HS_DNS_RC_PENDING,
	HS_DNS_RC_ALR_INIT,
	HS_DNS_RC_NOT_INIT,
	HS_DNS_RC_BAD_PARAMS,
	HS_DNS_RC_NO_MEM,
} hsdns_rc_t;

typedef void hs_dns_timer_cb_t(void *arg);
typedef long hs_dns_timer_start_t(unsigned long timeout_ms, void *arg, hs_dns_timer_cb_t *cb);
typedef void hs_dns_timer_stop_t(long timer_han);

typedef struct 
{
	int				cache_size;			// number of entries to keep in cache
	int				num_requests;		// number of concurrent requests to support
	unsigned long	server_ip;			// DNS server IP address
	hs_dns_timer_start_t *timer_start;	// timer start function
	hs_dns_timer_stop_t *timer_stop;	// timer stop function
} hsdns_cfg_t;

// run time parameters
typedef struct
{
	unsigned long	server_ip;			// DNS server ip address
} hsdns_params_t;

/*
 * Initialize DNS library
 */
extern int HsDnsInit(hsdns_cfg_t *cfg);

/*
 * Set parameters at run-time
 */
extern void HsDnsSetParams(hsdns_params_t *pPar);

/*
 * Shutdown DNS library
 */
extern void HsDnsCleanUp(void);

/*
 * Get IP address by name
 */
extern int HsDnsGetIpbyName(unsigned char *name, unsigned long *ipaddr);

#endif // _HS_DNS_IF_H
