/*
 * HS FTP Client Library
 * Recursive folder related operations
 * Developed by Hillstone Software
 * http://www.hillstone-software.com
 * support@hillstone-software.com
 */
#ifndef _HSFTP_RECURS_IF_H
#define _HSFTP_RECURS_IF_H

typedef enum
{
	HSFTP_RECURS_USR_EV_FOLDER_DELETE_FAILED,			// folder delete failed
	HSFTP_RECURS_USR_EV_FOLDER_DOWNLOAD_FAILED,		    // folder download failed
	HSFTP_RECURS_USR_EV_FOLDER_UPLOAD_FAILED,			// folder upload failed
	HSFTP_RECURS_USR_EV_FOLDER_DELETE_DONE,				// folder delete done, success
	HSFTP_RECURS_USR_EV_FOLDER_DOWNLOAD_DONE,			// folder download done, success
	HSFTP_RECURS_USR_EV_FOLDER_UPLOAD_DONE,				// folder upload done, success
	HSFTP_RECURS_USR_EV_FOLDER_DELETE_STATUS,			// reports start of stop of individual file or directory delete operation
	HSFTP_RECURS_USR_EV_FOLDER_DOWNLOAD_STATUS,	        // reports start of stop of individual file download operation
	HSFTP_RECURS_USR_EV_FOLDER_UPLOAD_STATUS,			// reports start of stop of individual file upload operation
	HSFTP_RECURS_USR_EV_FOLDER_PROGRESS,				// indicates operation progress
} hsftp_recurs_user_ev_t;

typedef enum
{
	HSFTP_OP_START	=1,									// start of individual operation 
	HSFTP_OP_DONE = 2,									// operation completed
} hsftp_op_t;

typedef void hsftp_recursive_callback_t(long ftp_session, int ev, long arg1, long arg2);

typedef enum
{
	HSFTP_RECURS_RC_OK			= 1,	// success
	HSFTP_RECURS_RC_BUSY		= 2,	// invalid state, recursive folder operation in progress
	HSFTP_RECURS_RC_NOTINIT		= 3,	// not initialised
	HSFTP_RECURS_RC_INVPAR		= 4,	// invalid parameters
	HSFTP_RECURS_RC_NOMEM		= 5,	// out of memory
	HSFTP_RECURS_RC_RCHDIR		= 6,	// remote change dir failed
	HSFTP_RECURS_RC_LIST		= 7,	// list failed
	HSFTP_RECURS_RC_FILE		= 8,	// file delete failed
	HSFTP_RECURS_RC_RMDIR		= 9,	// remove dir failed
	HSFTP_RECURS_RC_LCHDIR		= 10, 	// failed to change into local directory
	HSFTP_RECURS_RC_FOPEN		= 11, 	// failed to open file
	HSFTP_RECURS_RC_RMKDIR		= 12,	// failed to create folder at remote
	HSFTP_RECURS_RC_FSEND		= 13,	// failed to send file
	HSFTP_RECURS_RC_PATH		= 14,	// path too long
} hsftp_recurs_rc_t;

/* Initialise recursive operations module */
extern void HsFtpRecursInit(hsftp_recursive_callback_t *cb);

/* Release resources used by recursive folder operations module */
extern void HsFtpRecurseCleanUp(void);

/*
 * Feed events from core FTP library 
 */
extern void HsFtpRecursDoEvent(
	long 		session_handle,		// FTP session handle
	int 		ev, 				// event
	long 		arg1, 				// argument 1
	long 		arg2, 				// argument 2
	long 		arg3);				// argument 3

/*
 * Delete recursively complete folder with all subfolders and files
 */
extern int HsFtpRecursDeleteFolder(
	long 		session_handle, 	// FTP session handle
	unsigned char *foldername);		// folder to delete

/*
 * Download recursively complete folder with all subfolders and files
 */
extern int HsFtpRecursDownloadFolder(
	long 		session_handle, 	// FTP session handle
	unsigned char *foldername,		// folder to download
	int			overwrite);			// if true, overwrite local files and folders
									// if false, overwrite only if local and 
									// remote files size don't match

/*
 * Upload recursively complete folder with all subfolders and files
 */
extern int HsFtpRecursUploadFolder(
	long 		session_handle, 	// FTP session handle
	unsigned char *foldername);		// folder to upload

/*
 * Tick function
 */
extern void HsFtpRecursTick(void);

#endif /* _HSFTP_RECURS_IF_H */
