#include "stdafx.h"
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"
#include <stdlib.h>
#include <direct.h>
#include <stdio.h>
#include "HsSock_if.h"
#include "hsip_if.h"
#include "hstftp_if.h"
#include "hsdhcp_if.h"
#include <direct.h>
#include <process.h>
#include <Windowsx.h>
#include <Mmsystem.h>
#include "pcap.h"

#define WM_APP_TIMER	WM_USER + 1		// timer messages

#define MAXTIMERS		256

typedef struct
{
	int					allocated;
	int					enabled;
	void 				*arg;
	hs_sock_timer_cb_t 	*cb;
	UINT				timer_id;
} timerbock_t;

static timerbock_t	timers[MAXTIMERS] = {0};

// pcap local interface device list
pcap_if_t *alldevs = NULL;
pcap_t *adhandle = NULL;

unsigned char ascii_hex_eth[19] = {0};
unsigned char ethaddr[6];
unsigned char source_ipaddr[17] = {0};
unsigned char gw_ipaddr[17] = {0};
unsigned char mask[17] = {0};

static TIMECAPS timecaps;

TCHAR			szWindowClass[]		= " ";
HINSTANCE		hInst;
TCHAR			szTitle[] = "HsTCPIPv4 C Source Library: TFTP Demo (www.hillstone-software.com)";

HWND			hwnd_main;
HWND			dlg_main;
HBRUSH			hbrush1				= NULL;

char commercial_msg[] = "This is a demo application for HsTCPIPv4 embedded protocol stack. HSTFTP library is part of HsTCPIPv4.\n"
                        "It allows a programmer to upload and download files to and from remote TFTP server\nand also act as a server itself. "
                        "Further information: www.hillstone-software.com/hs_tcpipv4_details.htm\n"
                        "email: info@hillstone-software.com";


static	unsigned char current_dir[_MAX_PATH] = {0};
static	unsigned char strbuf[_MAX_PATH] = {0};
static	unsigned char current_filename[_MAX_PATH] = {0};

static int		num_events			= 0;

char *m_title = NULL;
char *m_prmt = NULL;
char *m_err = NULL;
char *m_hdr = "Error";

int server_started = 0;

unsigned char *initbuf;
DWORD ip_addr = 0xC0A80101;
WORD  ip_port = TFTP_PORT;

int tftp_initialised = 0;

int timer_id = 1;

unsigned char regdata[1024] = {0};

conf_t cfg = 
{
	{TFTP_PORT},				// tftp port
	{0xC0A80101},	// dest ip address
};

// server connections list
server_conn_list_t	conn_list = {0};

char *blocks[] = {
	"8",
	"16",
	"32",
	"64",
	"128",
	"256",
	"512",
	"1024",
	"1432",
	"2048",
	"8192",
	"16384",
	"32768",
	"65464",
	0
};

// allocate timer
static timerbock_t *alloc_timer(void)
{
	int i;

	for (i=0; i<MAXTIMERS; i++)
	{
		if (!timers[i].allocated)
		{
			timers[i].allocated = TRUE;
			return &timers[i];
		}
	}

	return NULL;
}

// free timer
static void free_timer(timerbock_t *p)
{
	p->allocated = FALSE;
}

/*
 * Initialise Combo box by adding list entries from array and setting initial selection
 * to index default_ind
 */
void InitComboControl(HWND wnd, char **buf, int dglid, int default_ind)
{
	int i = 0;

	while (buf[i])
	{
		SendDlgItemMessage(wnd, dglid, CB_ADDSTRING, 0, (long)buf[i++]);
	}

	SendMessage(GetDlgItem(wnd, dglid), CB_SETCURSEL, default_ind, 0);
}

HWND NewDialogWin(HINSTANCE hInst, HWND parent, DLGPROC proc, int dlg_id)
{
	HWND	handle;

	handle = CreateDialog(hInst, (LPCTSTR)dlg_id, parent, proc);

	ShowWindow	(handle, SW_SHOWDEFAULT);
    UpdateWindow(handle);

	return	handle;
}

ATOM BlRegisterClass(HINSTANCE	hInstance,
					UINT		style,
					WNDPROC		proc,
					int			cursor_id,
					HBRUSH		bkg_brush,
					LPCTSTR		lpszClassName,
					int			icon_id,
					int			menu_id)
{
	WNDCLASSEX	wcex;

	wcex.cbSize			= sizeof(WNDCLASSEX); 

	wcex.style			= style;
	wcex.lpfnWndProc	= proc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;

	wcex.hCursor		= 0;
	if (cursor_id)
	{
		wcex.hCursor	= LoadCursor(0, (const char *)cursor_id);
	}

	wcex.hbrBackground	= bkg_brush;
	wcex.lpszMenuName	= (LPCSTR)menu_id;
	wcex.lpszClassName	= lpszClassName;

	wcex.hIconSm		= 0; 
	if (icon_id)
	{
		wcex.hIcon		= LoadIcon(hInstance, (LPCTSTR)icon_id);
	}
	return RegisterClassEx(&wcex);
}

/* allocate a new server connection context  */
static server_conn_ctx_t *tftp_alloc_srvconn_ctx(void)
{
	server_conn_ctx_t	*pCtx;

	if (conn_list.count == TFTP_MAX_CTXS)
		return NULL;
	
	pCtx = (server_conn_ctx_t *)malloc(sizeof (server_conn_ctx_t));
	if (!pCtx) return NULL;
	
	memset(pCtx, 0, sizeof (server_conn_ctx_t));

	return pCtx;
}

/* free connection context */
static void tftp_free_srvconn_ctx(server_conn_ctx_t *pCtx)
{
	if (!pCtx) return;

	free(pCtx);
}

/* add context (to the tail of connections list) */
static int tftp_add_srvconn(server_conn_ctx_t *pCtx)
{
	if (!pCtx) return FALSE;

	pCtx->pNext = NULL;

	// if list is empty
	if (!conn_list.tail)
	{		
		conn_list.head = pCtx;
		conn_list.tail = pCtx;
		pCtx->pPrev = NULL;
	}
	else
	{
		conn_list.tail->pNext = pCtx;
		pCtx->pPrev = conn_list.tail;
		conn_list.tail = pCtx;	
	}

	conn_list.count++;

	return TRUE;
}

/* remove server connection from connection list */
static void tftp_remove_srvconn(server_conn_ctx_t *pCtx)
{
	if (pCtx->pNext)
		pCtx->pNext->pPrev = pCtx->pPrev;

	if (pCtx->pPrev)
		pCtx->pPrev->pNext = pCtx->pNext;
	
	if (pCtx == conn_list.tail)
		conn_list.tail = pCtx->pPrev;

	if (pCtx == conn_list.head)
		conn_list.head = pCtx->pNext;

	conn_list.count--;
}

BOOL InitListView (HWND hwndListView)
{
	int index;                      
	LV_COLUMN lvColumn;    
	
	TCHAR szString[6][40] = {TEXT("Type"),
                           TEXT("Operation"), 
                           TEXT("Filename"), 
                           TEXT("Remote IP"), 
                           TEXT("Block #"), 
                           TEXT("Bytes Transferred"),                            
	};
	
	int	clen[6] = {38,62, 140,100,100,150};

	int i = 0;

  	// Initialize the columns in the list view.
  	lvColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
  	lvColumn.fmt = LVCFMT_LEFT;

  	// Insert the five columns in the list view.
  	for (index = 0; index < 6; index++)
  	{
    	lvColumn.pszText = szString[index];
		lvColumn.cx = clen[index];
    	ListView_InsertColumn (hwndListView, index, &lvColumn);
  	}

	ListView_SetExtendedListViewStyle(hwndListView, LVS_EX_GRIDLINES | 
		                             LVS_EX_FULLROWSELECT);

  	return TRUE;
}

void ListViewAddSession(server_conn_ctx_t *pCtx, unsigned char *filename, unsigned long remote_ip)
{
	LVITEM 		item = {0};
  	unsigned char s[512];
  	int 		index;

	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST_SESS);

	if (!pCtx) return;

	memset(&item, 0, sizeof (LVITEM));

	item.mask = LVIF_TEXT | LVIF_PARAM;

	memset(s, 0, sizeof s);
	sprintf(s, "%s", pCtx->is_server_session ? "Serv" : "Clnt");	
	item.pszText = s;

	item.lParam = (LPARAM)pCtx;
	item.iItem = 0xffff;
	index = ListView_InsertItem(lvhan, (LPLVITEM)&item);

	memset(s, 0, sizeof s);
	sprintf(s, "%s", pCtx->is_send ? "Send" : "Receive");
	ListView_SetItemText(lvhan, index, 1, s);

	ListView_SetItemText(lvhan, index, 2, filename);

	memset(s, 0, sizeof s);
	HsSockInetNtoa(remote_ip, s);
	ListView_SetItemText(lvhan, index, 3, s);
	
	ListView_EnsureVisible(lvhan, index, 0);
}

void ListViewDeleteSession(server_conn_ctx_t *pCtx)
{
    LVFINDINFO findinfo = {0};
	int i;

	if (!pCtx) return;

    findinfo.flags = LVFI_PARAM;
    findinfo.lParam = (LPARAM)pCtx;
    i = ListView_FindItem(GetDlgItem(dlg_main, IDC_LIST_SESS), -1, &findinfo);

	if (i == -1) return;
    
	ListView_DeleteItem(GetDlgItem(dlg_main, IDC_LIST_SESS), i);
}

void ListViewSessionUpdateStats(server_conn_ctx_t *pCtx)
{
    LVFINDINFO findinfo = {0};
	int i;
	unsigned char s[80] = {0};
	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST_SESS);

	if (!pCtx) return;

    findinfo.flags = LVFI_PARAM;
    findinfo.lParam = (LPARAM)pCtx;
    i = ListView_FindItem(lvhan, -1, &findinfo);

	if (i == -1) return;

	memset(s, 0, sizeof (s));
	sprintf(s, "%u", pCtx->block_nr);
	ListView_SetItemText(lvhan, i, 4, s);

	memset(s, 0, sizeof (s));
	sprintf(s, "%u", pCtx->bytes_nr);
	ListView_SetItemText(lvhan, i, 5, s);
}

static void write_event(char *str)
{
	LVITEM 		item = {0};
  	unsigned char s[80];
  	int 		index;
  	int			*p;

	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST_EV);

	memset(&item, 0, sizeof (LVITEM));

	item.mask = LVIF_TEXT | LVIF_PARAM;

	memset(s, 0, sizeof s);
	sprintf(s, "%s",str);
	item.pszText = s;

	item.lParam = (LPARAM)1;
	item.iItem = 0xffff;
	index = ListView_InsertItem(lvhan, (LPLVITEM)&item);

	ListView_EnsureVisible(lvhan, index, 0);

	p = &num_events;

	if (*p < MAX_EVENTS)
	{
		(*p)++;
	}
	else
	{
		ListView_DeleteItem(lvhan, 0);
	}	
}

BOOL InitListAppView (HWND hwndListView)
{
	int index;                      
	LV_COLUMN lvColumn;    
	
	TCHAR szString[1][20] = {TEXT(" "),};
	
	int	clen[1] = {315};

	int i = 0;

  	// Initialize the columns in the list view.
  	lvColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
  	lvColumn.fmt = LVCFMT_LEFT;

  	// Insert the five columns in the list view.
  	for (index = 0; index < 1; index++)
  	{
    	lvColumn.pszText = szString[index];
		lvColumn.cx = clen[index];
    	ListView_InsertColumn (hwndListView, index, &lvColumn);
  	}

	ListView_SetExtendedListViewStyle(hwndListView, /*LVS_EX_GRIDLINES |*/ 
		                             LVS_EX_FULLROWSELECT);

  	return TRUE;
}


/*
 * Retrieve configuration from registry, if not found create
 */
void get_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp, size;

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					"Software\\Hillstone Software\\HS TFTP C Source Library Demo Application\\1.2",
            		0, KEY_QUERY_VALUE, &hKey );

    if (res == ERROR_SUCCESS)
    {    
		RegQueryValueEx(hKey, "RegData", 0, &dwDisp, (LPBYTE) regdata, &size);

   		memcpy(&cfg, regdata, sizeof(conf_t));
   		ip_addr = cfg.dest_ip;
   		ip_port = cfg.tftp_port;
   		
    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Hillstone Software\\HS TFTP C Source Library Demo Application\\1.2", 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{	
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/*
 * Update configuration in registry
 */
void update_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp;
	BOOL rs = FALSE;

	SendMessage(GetDlgItem(dlg_main, IDC_IPADDRESS1), IPM_GETADDRESS, 0,(LPARAM)&cfg.dest_ip);

	cfg.tftp_port = (WORD)GetDlgItemInt(dlg_main, IDC_EDIT_PORT, &rs, FALSE);
	if (!rs) ip_port = TFTP_PORT;

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					"Software\\Hillstone Software\\HS TFTP C Source Library Demo Application\\1.2",
            		0, KEY_ALL_ACCESS, &hKey );

    if (res == ERROR_SUCCESS)
    {    
   		memcpy(regdata, &cfg, sizeof(conf_t));
   		RegSetValueEx(hKey, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Hillstone Software\\HS TFTP C Source Library Demo Application\\1.2", 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/*
 * Error message handler
 */
static LRESULT CALLBACK DlgWinErr(HWND hDlg, UINT message, 
								 WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;

	switch (message)
	{
	case WM_INITDIALOG:
		SendMessage(hDlg, WM_SETTEXT, 0, (long)m_hdr);
		SendDlgItemMessage(hDlg, IDC_STATIC_TXT, WM_SETTEXT, 0, (long)m_err);
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_TXT))
		{
			SetTextColor((HDC)wParam, 0);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			EndDialog(hDlg, TRUE);
		}
		break;
	}
    return result;
}

/*
 * Handler for promotional dialog area at the bottom
 */
static LRESULT CALLBACK DlgWinMsg(HWND hDlg, UINT message, 
								 WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;

	switch (message)
	{
	case WM_INITDIALOG:
		SendMessage(hDlg, WM_SETTEXT, 0, (long)m_title);
		SendDlgItemMessage(hDlg, IDC_STATIC_G, WM_SETTEXT, 0, (long)m_prmt);
		SetDlgItemText(hDlg, IDC_EDIT, initbuf);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDCANCEL:
			EndDialog(hDlg, FALSE);
			return TRUE;

		case IDOK:
			GetDlgItemText(hDlg, IDC_EDIT, strbuf, sizeof (strbuf)-1);
			EndDialog(hDlg, TRUE);
		}
		break;
	}
    return result;
}

LRESULT GetStringDialog(HWND hDlg, char *title, char *prompt, unsigned char *initstr)
{
	INT_PTR result;

	memset(strbuf, 0, sizeof strbuf);

	m_title = title;
	m_prmt  = prompt;
	initbuf	= initstr;

	result = DialogBox(hInst, (LPCTSTR)IDD_DIALOG_MSG, hDlg, 
					  (DLGPROC)DlgWinMsg);

	return (LRESULT)result;
}

/*
 * Ask user for file name
 */
void GetFileName(HWND hDlg)
{
	OPENFILENAME ofn;       // common dialog box structure
	HWND hwnd =NULL;        // owner window

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFile = current_filename;
	//
	// Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
	// use the contents of szFile to initialize itself.
	//
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(current_filename);
	ofn.lpstrFilter = "All\0*.*\0Text\0*.TXT\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_NOCHANGEDIR;

	// Display the Open dialog box. 
	if (GetOpenFileName(&ofn)==TRUE) 
	{
		SetDlgItemText(hDlg, IDC_EDIT_FILENAME, current_filename);
	}
}

/*
 * Fetch next buffer to send
 */
unsigned char *get_next_send_block(server_conn_ctx_t *pCtx, int asked_len, int *given_len, int *info)
{
	unsigned char *bufp = NULL;
	int bytes_loaded = 0;
	int bytes_to_load = 0;
	DWORD dwBytes;

	/* if block is empty, try to load next block from file */
	if (pCtx->fblock_len == 0)
	{	
		ReadFile(pCtx->hFile, pCtx->fblock, FBLOCK_SZ, &dwBytes, NULL);
		bytes_loaded = (int)dwBytes;		
	
		if (bytes_loaded > 0)
		{
			pCtx->fblock_len = bytes_loaded;
			pCtx->fblock_ptr = 0;
		}
		else
		{
			CloseHandle(pCtx->hFile);
			pCtx->hFile = NULL;
			pCtx->fblock_ptr = 0;
			*given_len = 0;

			write_event("Send transfer complete");

			return NULL;		
		}		
	}

	bufp = &pCtx->fblock[pCtx->fblock_ptr];
	
	*given_len = (pCtx->fblock_len < asked_len) ? pCtx->fblock_len : asked_len;
	
	pCtx->fblock_len -= *given_len;
	pCtx->fblock_ptr += *given_len;

	pCtx->bytes_nr += *given_len;
	pCtx->block_nr++;

	ListViewSessionUpdateStats(pCtx);

	return bufp;	
}

/*
 * Fetch next buffer to receive
 */
unsigned char *get_next_receive_block(server_conn_ctx_t *pCtx, int asked_len, int *given_len)
{
	unsigned char	*bufp = NULL;
	int				bytes_written;
	DWORD			dwWritten;

	/* if reached end of receive buffer, dump to disk */
	if (asked_len > (FBLOCK_SZ - pCtx->rxbuf_len))
	{
		WriteFile(pCtx->hFile, pCtx->rxbuf, pCtx->rxbuf_len, &dwWritten, NULL);
		bytes_written = (int)dwWritten;

		if (bytes_written != pCtx->rxbuf_len)
		{
			CloseHandle(pCtx->hFile);
			pCtx->hFile = NULL;
			*given_len = 0;

			write_event("File write error");

			return NULL;
		}
		pCtx->rxbuf_ptr = 0;
		pCtx->rxbuf_len = 0;
	}

	bufp = &pCtx->rxbuf[pCtx->rxbuf_ptr];
	*given_len = asked_len;

	pCtx->rxbuf_ptr += *given_len;
	pCtx->rxbuf_len += *given_len;

	pCtx->bytes_nr += *given_len;
	pCtx->block_nr++;

	ListViewSessionUpdateStats(pCtx);

	return bufp;
}

unsigned char *tftp_get_buf_tx_cb(long handle, unsigned int *length, int *cmd)
{
	unsigned char *blockp = NULL;
	int len = 0;
	int info = 0;
	server_conn_ctx_t *pCtx = (server_conn_ctx_t *)handle;

	if (!pCtx) return NULL;

	blockp = get_next_send_block(pCtx, (int)(*length), &len, &info);
	if (!blockp)
	{
		// transfer complete
		hs_tftp_cleanup_ctx(pCtx);
	
		*length = 0;
		return NULL;
	}
	
	*length = len;
	return blockp;	
}

unsigned char *tftp_get_buf_rx_cb(long handle, unsigned int *length, int *cmd)
{
	unsigned char *blockp = NULL;
	int len = 0;
	server_conn_ctx_t *pCtx = (server_conn_ctx_t *)handle;

	if (!pCtx) return NULL;

	blockp = get_next_receive_block(pCtx, (int)(*length), &len);
	if (!blockp)
	{
		// block could not be written to file. Clean Up our context.
		hs_tftp_cleanup_ctx(pCtx);
	
		*length = 0;
		return NULL;
	}

	*length = len;
	return blockp;
}

/*
 * windows timer callback
 */
TIMERPROC TimerProc(HWND hwnd,  UINT uMsg, UINT_PTR idEvent,  DWORD dwTime)
{
	KillTimer(hwnd, idEvent);

	HsTftpTimerExpired((long)idEvent);
	return 0;
}

/* function prototype used by HS TFTP to start a timer 
 * secs - number of seconds to set timer to
 * handle - connection handle
 * return: timer handle or NULL if user code could not start timer */
long tftp_start_timer_cb(long handle, unsigned long secs)
{
	timer_id++;
	if (timer_id == 0) 
		timer_id = 1;

	if (SetTimer(hwnd_main, timer_id, (UINT)(secs * 1000), (TIMERPROC)TimerProc) == 0)
	{
		return 0;
	}

	return (long)timer_id;
}

/* function in used code used by HS TFTP to stop previously started timer */
/* Note that TFTP HS assumes that timer is stopped anyway if it has fired (expired) */
/* handle - timer handle */
void hs_tftp_stop_timer_cb(long handle)
{	
	KillTimer(hwnd_main, (UINT_PTR)handle);
}

/*
 * initiase TFTP library
 */
int init_tftp_library(void)
{
	tftp_init_t init = {0};
	int 	rc;

	init.get_tx_buffer 	= tftp_get_buf_tx_cb;
	init.get_rx_buffer 	= tftp_get_buf_rx_cb;
	init.start_timer_fn	= tftp_start_timer_cb;
	init.stop_timer_fn 	= hs_tftp_stop_timer_cb;

	init.max_blksize	= 65464;	// max blkoption option size value from RFC 2348
		
	rc = HsTftpInit(&init);

	if (rc == HS_TFTP_RC_OK)
		tftp_initialised = 1;

	return rc;
}

/*
 * Event calback from TFTP library
 */
long hs_tftp_ev_handler(long handle, int ev_code, long arg1, long arg2)
{
	unsigned char s[80] = {0};
	int len;
	unsigned char *errp = NULL;
	server_conn_ctx_t *pCtx;
	DWORD dwWritten;

	if (!tftp_initialised)
		return 0;

	pCtx = (server_conn_ctx_t *)handle;
	if (!pCtx) return 0;

	switch (ev_code)
	{
	case HS_TFTP_EV_ERR_TMOUT:
		write_event("Client session timeout");
		hs_tftp_cleanup_ctx(pCtx);
		break;

	case HS_TFTP_RX_COMPLETE:	
		write_event("Client receive transfer complete");
		if (pCtx->rxbuf_len > 0)
			WriteFile(pCtx->hFile, pCtx->rxbuf, pCtx->rxbuf_len, &dwWritten, NULL);
		hs_tftp_cleanup_ctx(pCtx);
		break;

	case HS_TFTP_EV_ERR_RXED:
		errp = (unsigned char *)arg2;

		len = strlen(errp);
		if (len > ((sizeof s)-1))
		{
			len = ((sizeof s)-1);
		}
		memcpy(s, errp, len);

		write_event(s);		
		hs_tftp_cleanup_ctx(pCtx);
		break;

	case HS_TFTP_EV_ERR_OACK:
		write_event("Client session closed: invalid OACK rxed");
		hs_tftp_cleanup_ctx(pCtx);		
		break;
	}

	return 0;
}

/* clean up application context */
static void hs_tftp_cleanup_ctx(server_conn_ctx_t *pCtx)
{
	if (!pCtx) return;

	ListViewDeleteSession(pCtx);

	if (pCtx->hFile) CloseHandle(pCtx->hFile);
	tftp_remove_srvconn(pCtx);
	tftp_free_srvconn_ctx(pCtx);
}

/*
 * Event calback from TFTP library (server mode)
 */
static long hs_tftp_server_ev_handler(long handle, int ev_code, long arg1, long arg2)
{
	unsigned char s[80] = {0};
	unsigned char *errp = NULL;
	server_conn_ctx_t *pCtx;
	DWORD dwWritten;

	if (!tftp_initialised)
		return 0;

	pCtx = (server_conn_ctx_t *)handle;
	if (!pCtx) return 0;

	switch (ev_code)
	{
	case HS_TFTP_EV_ERR_TMOUT:
		write_event("Servre session timeout");
		hs_tftp_cleanup_ctx(pCtx);
		break;

	case HS_TFTP_RX_COMPLETE:	
		write_event("Server receive transfer complete");
		if (pCtx->rxbuf_len > 0)
			WriteFile(pCtx->hFile, pCtx->rxbuf, pCtx->rxbuf_len, &dwWritten, NULL);
		hs_tftp_cleanup_ctx(pCtx);
		break;
	}

	return 0;
}

/* server mode - process file write request */
static void process_file_write_request(unsigned char *filename,  long arg2)
{
	int rc;
	unsigned char s[512] = {0};
	unsigned char ipstr[20] = {0};
	
	server_conn_ctx_t *pCtx;
	
	HsSockInetNtoa(arg2, ipstr);
	sprintf(s, "file write request from (%s) %s", ipstr, filename);
	write_event(s);
	
	pCtx = tftp_alloc_srvconn_ctx();
	if (!pCtx) 
	{
		write_event("rejected: no free contexts");
		HsTftpRejectRq(HS_TFTP_EV_WRITE_REQ, HS_TFTP_SERVER_ERR_USER, "no free contexts");	
		return;
	}

	pCtx->hFile = CreateFile(filename,    	// file to open
                   GENERIC_WRITE,			// open for writing
                   0,       				// no sharing
                   NULL,                  	// default security
                   OPEN_ALWAYS, 	      	// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 	// normal file
                   NULL); 

	if (pCtx->hFile == INVALID_HANDLE_VALUE)
		pCtx->hFile = NULL;

	if (!pCtx->hFile)
	{
		write_event("rejected: file open error");

		tftp_free_srvconn_ctx(pCtx);
		
		/* Reject request */
		HsTftpRejectRq(HS_TFTP_EV_WRITE_REQ, HS_TFTP_SERVER_ERR_FIO, NULL);
		return;
	}

	pCtx->is_server_session = TRUE;
	pCtx->is_send = FALSE;

	tftp_add_srvconn(pCtx);

	ListViewAddSession(pCtx, filename, (unsigned long)arg2);
	
	rc = HsTftpServerStartReceive(hs_tftp_server_ev_handler, &pCtx->tftp_handle, (long)pCtx);
	if (rc != HS_TFTP_RC_OK)
	{
		memset(s, 0, sizeof (s));
		sprintf(s, "Server Start Receive failed RC (%u)", rc);
		write_event(s);
		hs_tftp_cleanup_ctx(pCtx);
		return;
	}

	write_event("Server receive transfer started");
}

/* server mode - process file read request */
void process_file_read_request(unsigned char *filename, long arg2)
{
	int rc;
	long fsize = 0;
	unsigned char s[512] = {0};
	unsigned char ipstr[20] = {0};
	server_conn_ctx_t *pCtx;

	HsSockInetNtoa(arg2, ipstr);
	sprintf(s, "file read request from (%s) %s", ipstr, filename);
	write_event(s);
	
	pCtx = tftp_alloc_srvconn_ctx();
	if (!pCtx) 
	{
		write_event("rejected: no free contexts");
		HsTftpRejectRq(HS_TFTP_EV_READ_REQ, HS_TFTP_SERVER_ERR_USER, "no free contexts");	
		return;
	}
	
	pCtx->hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, 
						OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (pCtx->hFile == INVALID_HANDLE_VALUE)
		pCtx->hFile = NULL;

	fsize = (long)GetCompressedFileSize(filename, NULL);
	if ((!pCtx->hFile) || (!fsize))
	{
		write_event("rejected: file open error");
		HsTftpRejectRq(HS_TFTP_EV_READ_REQ, HS_TFTP_SERVER_ERR_FNOTFOUND, NULL);
		hs_tftp_cleanup_ctx(pCtx);
		return;
	}

	pCtx->fblock = pCtx->rxbuf;

	pCtx->is_server_session = TRUE;
	pCtx->is_send = TRUE;

	tftp_add_srvconn(pCtx);

	ListViewAddSession(pCtx, filename, (unsigned long)arg2);

	rc = HsTftpServerStartSend(hs_tftp_server_ev_handler, &pCtx->tftp_handle, (long)pCtx);
	if (rc != HS_TFTP_RC_OK)
	{
		memset(s, 0, sizeof (s));
		sprintf(s, "Server Start Send failed RC (%u)", rc);
		write_event(s);
		
		hs_tftp_cleanup_ctx(pCtx);
		return;
	}

	write_event("Server send transfer started");
}

/*
 * Event calback from TFTP library (server mode)
 */
static long hs_tftp_ev_server_handler(long handle, int ev_code, long arg1, long arg2)
{
	unsigned char *file;

	if (!tftp_initialised)
		return 0;

	if (!handle)
		return 0;

	switch (ev_code)
	{
	case HS_TFTP_EV_WRITE_REQ:
		file = (unsigned char *)arg1;
		process_file_write_request(file, arg2);
		break;

	case HS_TFTP_EV_READ_REQ:
		file = (unsigned char *)arg1;
		process_file_read_request(file, arg2);
		break;
	}

	return 0;
}

/*
 * remove path from filename
 */
static int strip_path(unsigned char *str)
{
	int len = strlen(str);
	int i;

	i = len-1;
	while (i)
	{
		if (str[i] == 0x5c)
		{
			return i+1; 
		}
		i--;	
	}

	return 0;
}

/* 
 * Client get file from TFTP server
 */
void ClientGetFile(HWND hDlg)
{
	unsigned char s[_MAX_PATH+1] = {0};
	int rc;
	FILE *fd = NULL;
	DWORD dest_ip;
	int i;
	server_conn_ctx_t *pCtx;
	BOOL res;
	unsigned short int blksize;
	char blksz_str[20];

	if (!tftp_initialised)
	{
		write_event("Client: HS TFTP failed to initialise");
		return;
	}

	GetDlgItemText(hDlg, IDC_EDIT_FILENAME, s, _MAX_PATH);
	if (strlen(s) == 0)
	{
		write_event("Client: could not open file for transfer");
		return;	
	}

	memcpy(current_filename, s, _MAX_PATH);

	SendMessage(GetDlgItem(hDlg, IDC_IPADDRESS1), IPM_GETADDRESS, 0,(LPARAM)&dest_ip);  

	ip_port = (WORD)GetDlgItemInt(hDlg, IDC_EDIT_PORT, &res, FALSE);
	if (!res) ip_port = TFTP_PORT;

	i = strip_path(current_filename);

	pCtx = tftp_alloc_srvconn_ctx();
	if (!pCtx) 
	{
		write_event("Client: cannot get file: no free contexts");
		tftp_free_srvconn_ctx(pCtx);
		return;
	}

	pCtx->hFile = CreateFile(&current_filename[i], // file to open
                   GENERIC_WRITE,			// open for writing
                   0,       				// no sharing
                   NULL,                  	// default security
                   OPEN_ALWAYS, 	      	// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 	// normal file
                   NULL); 

	if (pCtx->hFile == INVALID_HANDLE_VALUE)
		pCtx->hFile = NULL;

	if (!pCtx->hFile)
	{
		write_event("Client: receve failed: file open error");

		tftp_free_srvconn_ctx(pCtx);
		return;
	}

	pCtx->is_server_session = FALSE;
	pCtx->is_send = FALSE;

	tftp_add_srvconn(pCtx);

	ListViewAddSession(pCtx, &current_filename[i], dest_ip);

	blksize = 0;	// by default, let HS TFTP use default block size, without option negotiation

	// but if option negotiation is enabled, use user selected block size
	if (IsDlgButtonChecked(hDlg, IDC_CHECK_BLK) == BST_CHECKED)
	{
		GetDlgItemText(hDlg, IDC_COMBO_BLK, blksz_str, sizeof (blksz_str));
		blksize = (unsigned short int)atoi(blksz_str);
	}

	rc = HsTftpTransfer(TFTP_OP_GET_FILE, dest_ip, 
				&current_filename[i], hs_tftp_ev_handler, ip_port, blksize, &pCtx->tftp_handle, (long)pCtx);

	if (rc != HS_TFTP_RC_OK)
	{
		memset(s, 0, sizeof s);
		sprintf(s,"HsTftpTransfer failed. Error %d",rc);
		write_event(s);
		
		hs_tftp_cleanup_ctx(pCtx);
		return;
	}

	write_event("Client receive transfer started");
}

/* 
 * Client send file to TFTP server
 */
void ClientSendFile(HWND hDlg)
{
	unsigned char s[_MAX_PATH+1] = {0};
	int rc;
	DWORD dest_ip;
	int i;
	long fsize;
	server_conn_ctx_t *pCtx;
	BOOL res;
	unsigned short int blksize;
	char blksz_str[20] = {0};

	if (!tftp_initialised)
	{
		write_event("Client: HS TFTP failed to initialise");
		return;
	}

	GetDlgItemText(hDlg, IDC_EDIT_FILENAME, s, _MAX_PATH);
	if (strlen(s) == 0)
	{
		write_event("Client: cannot send - no filename");
		return;
	}

	pCtx = tftp_alloc_srvconn_ctx();
	if (!pCtx) 
	{
		write_event("Client: cannot send file: no free contexts");
		return;
	}

	pCtx->is_server_session = FALSE;
	pCtx->is_send = TRUE;
	pCtx->fblock = pCtx->rxbuf;

	tftp_add_srvconn(pCtx);
	
	memcpy(current_filename, s, _MAX_PATH);

	SendMessage(GetDlgItem(hDlg, IDC_IPADDRESS1), IPM_GETADDRESS, 0,(LPARAM)&dest_ip);  

	ip_port = (WORD)GetDlgItemInt(hDlg, IDC_EDIT_PORT, &res, FALSE);
	if (!res) ip_port = TFTP_PORT;

	i = strip_path(current_filename);

	ListViewAddSession(pCtx, &current_filename[i], dest_ip);

	pCtx->hFile = CreateFile(current_filename, GENERIC_READ, FILE_SHARE_READ, NULL, 
						OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (pCtx->hFile == INVALID_HANDLE_VALUE)
		pCtx->hFile = NULL;

	fsize = (long)GetCompressedFileSize(s, NULL);
	if (!fsize) pCtx->hFile = NULL;

	if ((!pCtx->hFile) || (!fsize))
	{
		write_event("Client: cannot send file: file open error");
		hs_tftp_cleanup_ctx(pCtx);
		return;
	}

	blksize = 0;	// by default, let HS TFTP use default block size, without option negotiation

	// but if option negotiation is enabled, use user selected block size
	if (IsDlgButtonChecked(hDlg, IDC_CHECK_BLK) == BST_CHECKED)
	{
		GetDlgItemText(hDlg, IDC_COMBO_BLK, blksz_str, sizeof (blksz_str));
		blksize = (unsigned short int)atoi(blksz_str);
	}

	rc = HsTftpTransfer(TFTP_OP_SEND_FILE, dest_ip, 
				&current_filename[i], hs_tftp_ev_handler, ip_port, blksize, &pCtx->tftp_handle, (long)pCtx);

	if (rc != HS_TFTP_RC_OK)
	{
		memset(s, 0, sizeof s);
		sprintf(s,"Client: cannot send file: HS TFTP Error: %s", HsTftpErrStr(rc));
		write_event(s);
		hs_tftp_cleanup_ctx(pCtx);
		return;
	}

	write_event("Client send transfer started");
}

/* Start server mode */
void StartServer(void)
{
	int rc;
	unsigned char s[80] = {0};

	if (!tftp_initialised)
	{
		rc = init_tftp_library();
		if (rc != HS_TFTP_RC_OK)
		{
			memset(s, 0, sizeof s);
			sprintf(s,"HsTFTP init failed. Error %d",rc);
			write_event(s);	
			return;
		}
	}	

	if (!server_started)
	{
		rc = HsTftpStartServer(hs_tftp_ev_server_handler, 65464, TFTP_PORT);
		if (rc != HS_TFTP_RC_OK)
		{		
			memset(s, 0, sizeof s);
			sprintf(s,"Server failed to start. HS TFTP Error: %d", rc);
			write_event(s);
			return;
		}
		server_started = TRUE;
	}

	write_event("Server mode started OK");
}

/*
 * Abort current transfer
 */
void Abort(HWND hDlg)
{
	LRESULT iSelect;
	LVITEM	item = {0};
	server_conn_ctx_t *pCtx;

	if (!tftp_initialised)
	{
		write_event("HS TFTP failed to initialize");
		return;
	}

	iSelect=SendMessage(GetDlgItem(dlg_main, IDC_LIST_SESS), LVM_GETNEXTITEM, -1, LVNI_FOCUSED);
	if (iSelect == -1)
	{
		return;
	}
			
	item.mask = LVIF_PARAM;
	item.iItem = (int)iSelect;
	ListView_GetItem(GetDlgItem(dlg_main, IDC_LIST_SESS), &item);
	pCtx = (server_conn_ctx_t *)item.lParam;
	
	write_event("File transfer session aborted");

	HsTftpAbort(pCtx->tftp_handle);

	hs_tftp_cleanup_ctx(pCtx);	
}

void GetCurDir(HWND hDlg)
{
	int rc;

	while (1)
	{
		rc = GetStringDialog(hDlg, "Change Working Directory", "Enter New Directory :",
			                current_dir);
		if (rc)
		{
			if (chdir(strbuf) == 0)
			{
				getcwd( current_dir, _MAX_PATH );
				SetDlgItemText(hDlg, IDC_EDIT_CDIR, current_dir);
				break;
			}
			m_err ="Invalid Directory!";
			DialogBox(hInst, (LPCTSTR)IDD_DIALOG_ERR, hDlg, 
					 (DLGPROC)DlgWinErr);
		}
		else break;
	}
}

static LRESULT CALLBACK DlgWinAbout(HWND hDlg, UINT message, 
						  	WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;
	unsigned char s[400] = {0};

	switch (message)
	{
	case WM_INITDIALOG:
		sprintf(s, "\n\nDemo application for HS TFTP "HS_VERSION" Source Code Library in C\n"
				"\nTo purchase HS TFTP module\n"
				"(binaries, full source, documentation, examples):\n"
				"\nhttp://www.hillstone-software.com/hs_tftp_buy.htm\n"
				"\nemail: info@hillstone-software.com\n\n"
				"Tel. +353 87 988 1568\n");
		
		SendDlgItemMessage(hDlg, IDC_STATIC_AB, WM_SETTEXT, 0, (long)s);
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_AB))
		{
			SetTextColor((HDC)wParam, 0);
			SetBkColor ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		break;
	
	case WM_COMMAND:
		break;

	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
		DestroyWindow(hDlg);
		break;

	default:
		break;
	}
    return result;
}

/*
 * Process menu
 */
LRESULT CALLBACK ProcessMainMenu(HWND hWnd, int wmEvent) 
{
	switch (wmEvent)
	{
	case ID_TOOLS_CHANGEDIRECTORY:
		GetCurDir(dlg_main);
		break;

	case IR_MENU_SEND:
		ClientSendFile(dlg_main);
		break;

	case ID_TOOLS_ABORTTRANSFER:
		Abort(dlg_main);
		break;

	case ID_FILE_RECEIVEFILE:
		ClientGetFile(dlg_main);
		break;

	case ID_FILE_EXIT:
		DestroyWindow(hwnd_main);
		break;

	case ID_ABOUT:
		NewDialogWin(hInst, dlg_main, DlgWinAbout, IDD_DIALOG_ABOUT);
		break;
	}

	return FALSE;
}

static unsigned char hex2byte(unsigned char *buf)
{
	unsigned char hex[] = "0123456789ABCDEF";
	unsigned char b = 0;
    unsigned char b1, b2;
	int i;

    b1 = toupper(buf[0]);
    b2 = toupper(buf[1]);

	for (i = 0; i<16; i++)
	{
		if (b1 == hex[i])
		{
			b &= 0x0f;
			b = (unsigned char)((i << 4) & 0xf0);
			break;
		}
	}

	for (i = 0; i<16; i++)
	{
		if (b2 == hex[i])
		{
			b &= 0xf0;
			b |= (unsigned char)(i & 0x0f);
			break;
		}
	}
	
	return b;
}

// Convert ascii hex ethernet address in the form 00:00:00:00:00:00 to binary 6 byte buffer
static void convert_hex_ethaddr_to_binary(unsigned char *hexbuf, unsigned char *binbuf)
{
	int i, n, maxlen, done;

	maxlen = 17;

	if (strlen(hexbuf) != 17)
		return;

	done = FALSE;
	i = 0; n = 0;
	while (!done)
	{
		binbuf[i] = hex2byte(&hexbuf[n]);

		i++;
		n += 3;

		if (i == 6) 
			done = TRUE;
	}
}

// multimedia timer callback
void CALLBACK mmTimeProc(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	PostMessage(hwnd_main, WM_APP_TIMER, 0, (LPARAM)dwUser);
}

// Process timer message
static void process_timer(LPARAM lParam)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)lParam;

	if (!pTmrBlk) return;

	if (!pTmrBlk->allocated)
		return;

	if (!pTmrBlk->enabled)
		return;

	pTmrBlk->cb(pTmrBlk->arg);

	free_timer(pTmrBlk);
}

// start timer function
static long sock_timer_start(unsigned long timeout_ms, void *arg, hs_sock_timer_cb_t *cb)
{
	MMRESULT res;
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)alloc_timer();
	if (!pTmrBlk) return 0;

	pTmrBlk->arg = arg;
	pTmrBlk->cb = cb;
	pTmrBlk->enabled = TRUE;

	res = timeSetEvent((UINT)timeout_ms, timecaps.wPeriodMin, mmTimeProc, (DWORD)pTmrBlk, TIME_ONESHOT);
	if (res == 0) 
	{	
		free_timer(pTmrBlk);
		return 0;
	}

	pTmrBlk->timer_id = (UINT)res;

	return (long)pTmrBlk;
}

// stop timer function
static void sock_timer_stop(long timer_han)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *) timer_han;

	if (!pTmrBlk) return;

	if (!pTmrBlk->enabled) return;

	timeKillEvent(pTmrBlk->timer_id);
	pTmrBlk->enabled = FALSE;
}

// transmit over physical layer (ethernet)
static void sock_drv_tx_fn(unsigned char *pkt, int len)
{
	if (!adhandle) return;

	/* Send down the packet */
	if (pcap_sendpacket(adhandle,	// Adapter
		pkt,						// buffer with the packet
		len							// size
		) != 0)
	{
		return;
	}
}

// callback used by HSTCP to get number of millisecond ticks since bootup
// Used for generating pseudo random initial sequence number
// if callback pointer not initialized, HSTCP will use default 
// pseudo random number generator to produce initial sequence numbers
static unsigned long hs_sock_get_ticks(void)
{
	return (unsigned long)GetTickCount();
}

// Callback used by the stack whenever it needs a random 32 bit number
static unsigned long hs_sock_get_rand32(void)
{
	return (unsigned long)rand();
}

// global socket events
static long hs_sock_global_event(long user_handle, int ev, long arg1, long arg2, long arg3, long arg4)
{
	hs_dhcp_update_t *p;
	char str[17];

	switch (ev)
	{

	// have been aquired by HSDHCP
	case HS_SOCK_EV_DHCP_UPDATE:
		p = (hs_dhcp_update_t *)arg1;
		if (!p) return 0;

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->local_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS2, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->router_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_GW, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->subnet_mask, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_MASK, str);

		break;
	}

	return 0;
}

// Initialize
static void initialize(HWND hDlg)
{
	int rc, inum, i;

	unsigned char ascii_hex_eth[19] = {0};
	unsigned char ethaddr[6];

	unsigned char source_ipaddr[17] = {0};
	unsigned char gw_ipaddr[17] = {0};
	unsigned char mask[17] = {0};
	unsigned char dns_ipaddr[17] = "0.0.0.0";

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	hssock_init_t sockinit = {0};

	if (adhandle)
		pcap_close(adhandle);
			
	inum = (int)SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_GETCURSEL, 0, 0);
	if (inum == CB_ERR)
	{
		write_event("Local adapter not selected from list");
		return;
	}

	GetDlgItemText(hDlg, IDC_IPADDRESS2, source_ipaddr, sizeof (source_ipaddr) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_GW, gw_ipaddr, sizeof (gw_ipaddr) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_MASK, mask, sizeof (mask) - 1);
		
	GetDlgItemText(hDlg, IDC_EDIT_MAC, ascii_hex_eth, sizeof (ascii_hex_eth) - 1);

	convert_hex_ethaddr_to_binary(ascii_hex_eth, ethaddr);
				
	/* Jump to the selected adapter */
	for(d=alldevs, i=0; i< inum ;d=d->next, i++);

	if ((adhandle= pcap_open_live(d->name,	// name of the device
					 65536,					// portion of the packet to capture. 
											// 65536 grants that the whole packet will be captured on all the MACs.
					 1,						// promiscuous mode (nonzero means promiscuous)
					 1,						// read timeout
					 errbuf					// error buffer
					 )) == NULL)
	{
		write_event("Unable to open the adapter : not supported by WinPcap");
		return;
	}

	sockinit.dhcp_client = (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED) ? 1: 0;

	if (sockinit.dhcp_client)
		strcpy(source_ipaddr, "0.0.0.0");

	sockinit.source_ip_str = source_ipaddr;
	sockinit.gateway_ip_str = gw_ipaddr;
	sockinit.network_mask = mask;
	sockinit.dns_server_ip_str = dns_ipaddr;	
	sockinit.local_eth = ethaddr;
	sockinit.timer_start_fn = sock_timer_start;
	sockinit.timer_stop_fn = sock_timer_stop;
	sockinit.drv_tx_fn = sock_drv_tx_fn;
	sockinit.log_fn = write_event;

	sockinit.get_ticks_fn = hs_sock_get_ticks;
	sockinit.seed = (unsigned long)GetTickCount();
	sockinit.get_rand32 = hs_sock_get_rand32;
	sockinit.global_ev_fn = hs_sock_global_event;

	rc = HsSockInit(&sockinit);
	if (rc != HS_SOCK_RC_OK)
	{
		sprintf(str, "HsSockInit failed, rc=%s", HsSockGetRcString(rc));
		write_event(str);
		return;
	}

	write_event("HsTCPIPv4 Initialized");

	StartServer();
}

/*
 * Main dialog handler
 */
LRESULT CALLBACK DlgWin(HWND hDlg, UINT message, 
						  WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;
    char windir[_MAX_PATH] = {0};
    char expl_path[_MAX_PATH] = {0};

	int i;

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	switch (message)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_IPADDRESS2, "192.168.1.51");
		SetDlgItemText(hDlg, IDC_EDIT_MAC, "00:1e:68:a4:fb:15");

		SetDlgItemText(hDlg, IDC_IPADDRESS_GW, "192.168.1.254");
		SetDlgItemText(hDlg, IDC_IPADDRESS_MASK, "255.255.255.0");

		CheckDlgButton(hDlg, IDC_CHECK_DHCP, BST_CHECKED);
		EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS2), FALSE);

		/* Retrieve the device list */
		if(pcap_findalldevs(&alldevs, errbuf) == -1)
		{
			sprintf(str, "Error in pcap_findalldevs: %s", errbuf);
			write_event(str);
			return TRUE;
		}

		i = 0;
		for(d=alldevs; d; d=d->next)
		{			
			sprintf(str, "[%d] %s", ++i, d->description);
			SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_ADDSTRING, 0, (long)str);			
		}	

		SendMessage(GetDlgItem(hDlg, IDC_COMBO_ETHIF), CB_SETCURSEL, 0, 0);


		
		SendDlgItemMessage(hDlg, IDC_STATIC_MSG, WM_SETTEXT, 0, (long)commercial_msg);
		
		SetDlgItemText(hDlg, IDC_EDIT_CDIR, current_dir);

		SendMessage(GetDlgItem(hDlg, IDC_IPADDRESS1), IPM_SETADDRESS, 0, (LPARAM)ip_addr);

		SetDlgItemInt(hDlg, IDC_EDIT_PORT, (UINT)ip_port, FALSE);

		dlg_main = hDlg;

		InitListAppView(GetDlgItem(hDlg, IDC_LIST_EV));		
		InitListView(GetDlgItem(hDlg, IDC_LIST_SESS));

		InitComboControl(hDlg, &blocks[0], IDC_COMBO_BLK, 8);

		CheckDlgButton(hDlg, IDC_CHECK_BLK, BST_UNCHECKED);
		EnableWindow(GetDlgItem(hDlg, IDC_COMBO_BLK), FALSE);

		srand( (unsigned)time( NULL ) );
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if (((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_STAT)) ||
			((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_SESSL)) ||
			((HWND)lParam == GetDlgItem(hDlg, IDC_EDIT_CDIR)))
		{
			SetTextColor((HDC)wParam, 0);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		else if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_MSG))
		{
			SetTextColor((HDC)wParam, BLUE);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}		
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_CHECK_DHCP:
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED)
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS2), FALSE);
			else
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS2), TRUE);
			break;
		
		case IDC_BUTTON_SHUT:
			HsSockCleanUp();
			HsTftpDestroy();

			if (adhandle)
			{
				pcap_close(adhandle);
				adhandle = NULL;
			}
			
			write_event("HsSock de-initialized OK");		
			break;

		case IDC_BUTTON_INIT1:
			initialize(hDlg);
			break;
		
		case IDC_BUTTON_DIR:
			GetCurDir(hDlg);
			break;

		case IDC_BUTTON_FILE:
			GetFileName(hDlg);
			break;

		case IDC_BM_SEND:
			ClientSendFile(hDlg);
			break;

		case IDC_BM_ABORT:
			Abort(hDlg);
			break;

		case IDC_BM_GET:
			ClientGetFile(hDlg);
			break;

		case IDC_BM_ABOUT:
			NewDialogWin(hInst, hDlg, DlgWinAbout, IDD_DIALOG_ABOUT);
			break;

		case IDC_CHECK_BLK:
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_BLK) == BST_CHECKED)
				EnableWindow(GetDlgItem(hDlg, IDC_COMBO_BLK), TRUE);
			else
				EnableWindow(GetDlgItem(hDlg, IDC_COMBO_BLK), FALSE);
			break;
		}
		break;
	}
    return result;
}

/*
 * Main window message handler
 */
LRESULT CALLBACK WndProc(HWND		hWnd, 
						 UINT		message, 
						 WPARAM		wParam, 
						 LPARAM		lParam)
{
	static INITCOMMONCONTROLSEX icex;
	server_conn_ctx_t *pCtx, *pCtx1;

	switch (message) 
	{
	case WM_CREATE:
		icex.dwSize = sizeof(icex);
		icex.dwICC = ICC_DATE_CLASSES;

		InitCommonControlsEx(&icex);

		InitCommonControls(); 

		dlg_main = NewDialogWin(hInst, hWnd, DlgWin, IDD_DIALOG_MAIN);
		break;

	case WM_APP_TIMER:
		process_timer(lParam);
		break;

	case WM_COMMAND:
		ProcessMainMenu(hWnd, LOWORD(wParam));
		break;

	case WM_DESTROY:
		if (hbrush1)
		{
			SelectObject(GetWindowDC(hWnd), hbrush1);
			DeleteObject(hbrush1);
		}

		pCtx = conn_list.head;
		while (pCtx)
		{
			pCtx1 = pCtx->pNext;
			hs_tftp_cleanup_ctx(pCtx);
			pCtx = pCtx1;
		}

		if (tftp_initialised)
			HsTftpDestroy();

		update_config();

		HsSockCleanUp();

		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
   
	return 0;
}

/*
 * Init Application Instance
 */
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, HWND *hwnd)
{
	HWND hWnd;

	hInst = hInstance;

	hWnd = CreateWindow(szWindowClass, 
					   szTitle, 
					   WS_OVERLAPPED | WS_SYSMENU,
					   0, 
					   0, 
					   MAINWIDTH,
					   MAINDEPTH, 
					   NULL, LoadMenu(hInstance, (LPCTSTR)IDR_MENU1), 
					   hInstance, NULL);

	if (!hWnd)
	{
		return FALSE;
	}
	*hwnd = hWnd;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;
}

/*
 * entry point
 */
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG		msg;
	HACCEL	hAccelTable;

	unsigned char *pkt_data;
	struct pcap_pkthdr *header;
	MMRESULT res;
	int r;

	if( getcwd( current_dir, _MAX_PATH ) == NULL )
	{
		MessageBox(0, "Cannot get current working directory", 0, 0);
		return 0;
	}

	res = timeGetDevCaps(&timecaps, sizeof (TIMECAPS));
	if (res == TIMERR_NOERROR)
	{			
		timeBeginPeriod((UINT)timecaps.wPeriodMin);
	}

	get_config();

	hbrush1 = CreateSolidBrush(RGB(255, 255, 255));

	BlRegisterClass(hInstance, CS_HREDRAW | CS_VREDRAW, WndProc, (int)IDC_ARROW,
				   (HBRUSH)(COLOR_APPWORKSPACE+1), szWindowClass, IDI_ICON2, IDR_MENU1);

	if (!InitInstance (hInstance, nCmdShow, &hwnd_main)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)0);

	while (1) 
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{
				if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
			else
			{
				break;
			}
		}
		else
		{
			Sleep(1);

			if (adhandle)
			{
				r = pcap_next_ex( adhandle, &header, &pkt_data);
				while (r > 0)
				{
					HsIpReceivePacket(pkt_data, (int)header->caplen);
					
					r = pcap_next_ex( adhandle, &header, &pkt_data);
				}
			}			
		}		
	}

	if (alldevs)
		pcap_freealldevs(alldevs);

	if (adhandle)
		pcap_close(adhandle);
	
	return msg.wParam;	
}



